import torch
import albumentations
from albumentations.pytorch import ToTensorV2

DEVICE = "cuda" if torch.cuda.is_available() else 'cpu'
LEARNING_RATE = 1e-4
WEIGHT_DECAY = 5e-4
BATCH_SIZE = 12
NUM_EPOCHS = 100
NUM_WORKERS = 6
CHECKPOINT_FILE = ".tar"
PIN_MEMORY = True
SAVE_MODEL = True
LOAD_MODEL = True
MAINTAIN_RATIO = False
IMAGE_SIZE = 224

#Data augmentation for images
train_transforms = albumentations.Compose(
    [
        albumentations.Resize(width=IMAGE_SIZE, height=IMAGE_SIZE),
        albumentations.RandomCrop(width=200, height=200, ),
        albumentations.HorizontalFlip(p=0.5),
        albumentations.Blur(p=0.3),
        albumentations.CLAHE(p=0.3),
        albumentations.ColorJitter(p=0.3),
        albumentations.CoarseDropout(max_holes=12,max_height=5,max_width=5,p=0.3), #随机给图像上小黑点
        albumentations.Affine(rotate=9,p=0.2,shear=10),
        albumentations.RandomCrop(width=180,height=180,),
        albumentations.Normalize(mean = [0.3199,0.2240, 0.1609 ],
                                 std =  [0.3020, 0.2180,0.1740],
                                 max_pixel_value=255.0),
        ToTensorV2(),
    ]
)


val_transforms = albumentations.Compose(
    [
        albumentations.Resize(width=180, height=180),
        #albumentations.Resize(width=IMAGE_SIZE, height=IMAGE_SIZE),
        #albumentations.RandomCrop(width=200, height=200),
        #albumentations.CenterCrop(width=180, height=180),
        albumentations.Normalize(mean=[0.3199, 0.2240, 0.1609],
                                 std=[0.3020, 0.2180, 0.1740],
                                 max_pixel_value=255.0),
        ToTensorV2(),
    ]
)