import baseline.config as config
import os
import albumentations
from albumentations.pytorch import ToTensorV2
import pandas as pd
import numpy as np
from torch.utils.data import DataLoader , Dataset
from PIL import Image
from tqdm import tqdm
from .preprocess_images import resize_maintain_aspect

class Breast_Dataset_with_Path(Dataset):#make datasets

    def __init__(self , annotation_lines  , train= True, transform = None , maintain_ratio = False):
        super(Breast_Dataset_with_Path, self).__init__()
        self.annotation_lines = annotation_lines
        self.transform = transform
        self.train = train
        self.maintain_ratio = maintain_ratio

    def __len__(self):
        return len(self.annotation_lines)

    def __getitem__(self, index):

        lesion = self.annotation_lines[index].split(';')[-3].split("\n")[0]  # get the datasets path
        lesion_withCDFI = self.annotation_lines[index].split(';')[-2].split("\n")[0]
        filename =self.annotation_lines[index].split('/')[3] #get patient ID

        lesion = "G:/Multimodal_BR3_classfication/" + lesion.split('.')[1]+".tif"
        lesion_withCDFI = "G:/Multimodal_BR3_classfication" + lesion_withCDFI.split('.')[1]+".tif"

        image = Image.open(lesion).convert("RGB")
        image_resize_180 = image.resize((180 , 180), Image.BICUBIC)

        image_CDFI = Image.open(lesion_withCDFI).convert("RGB")
        image_CDFI_resize_180 = image_CDFI.resize((180, 180), Image.BICUBIC)
        if self.maintain_ratio:
            image = resize_maintain_aspect(image,desired_size=config.IMAGE_SIZE)
            image_CDFI = resize_maintain_aspect(image_CDFI,desired_size=config.IMAGE_SIZE)
        image = np.array(image)
        image_CDFI = np.array(image_CDFI)
        label = int(self.annotation_lines[index].split(';')[0])
        if self.transform:
            image = self.transform(image=image)["image"]
            image_CDFI = self.transform(image = image_CDFI)["image"]

        return image, image_CDFI, label , filename , np.array(image_resize_180), np.array(image_CDFI_resize_180)



class Breast_Dataset(Dataset):
    def __init__(self , annotation_lines  , train= True, transform = None , maintain_ratio = False):
        super(Breast_Dataset, self).__init__()
        self.annotation_lines = annotation_lines
        self.transform = transform
        self.train = train
        self.maintain_ratio = maintain_ratio

    def __len__(self):
        return len(self.annotation_lines)

    def __getitem__(self, index):

        lesion = self.annotation_lines[index].split(';')[-3].split("\n")[0]  # get the datasets path
        lesion_withCDFI = self.annotation_lines[index].split(';')[-2].split("\n")[0]
        filename =self.annotation_lines[index].split('/')[3] #get patient ID

        lesion = "G:/Multimodal_BR3_classfication/" + lesion.split('.')[1]+".tif"
        lesion_withCDFI = "G:/Multimodal_BR3_classfication" + lesion_withCDFI.split('.')[1]+".tif"

        image = Image.open(lesion).convert("RGB")
        image_CDFI = Image.open(lesion_withCDFI).convert("RGB")
        if self.maintain_ratio:
            image = resize_maintain_aspect(image,desired_size=config.IMAGE_SIZE)
            image_CDFI = resize_maintain_aspect(image_CDFI,desired_size=config.IMAGE_SIZE)
        image = np.array(image)
        image_CDFI = np.array(image_CDFI)
        label = int(self.annotation_lines[index].split(';')[0])
        if self.transform:
            image = self.transform(image=image)["image"]
            image_CDFI = self.transform(image = image_CDFI)["image"]

        return image, image_CDFI, label , filename

if __name__ == "__main__":
    """
    Test works
    """

    breast_path = "G:/Multimodal_BR3_classfication/datasets_index/train1_4a.txt"
    with open(breast_path, "r", encoding='gbk') as f:
        lines = f.readlines()
    dataset = Breast_Dataset(
        annotation_lines= np.array(lines),
        transform = config.train_transforms,
    )
    loader = DataLoader(
        dataset = dataset, batch_size=12,num_workers=6,shuffle=True,pin_memory=True
    )
    for image,image_CDFI,label ,filename in tqdm(loader):
        print(image.shape)
        print(image_CDFI.shape)
        print(label.shape)
        import sys
        sys.exit()