import torch
from torch import nn,optim
import os
import random
import pandas as pd
from sklearn.model_selection import train_test_split
import numpy as np
from Nets import resnet
from Nets import SEnet
from Nets import vgg16
import baseline.config as config
from torch.utils.data import DataLoader
from tqdm import tqdm
from Nets.EfficienNet import Multimodel_efficientNet_v2_s
from baseline.dataset import Breast_Dataset
from torchvision.utils import save_image
from baseline.utils import (
    load_checkpoint,
    save_checkpoint,
    check_accuracy,
    make_prediction,
)

def train_one_epoch(loader , model , optimizer , loss_fn , scaler, device):
    losses = []
    loop = tqdm(loader)
    for batch_index, (data, data_CDFI , targets, _) in enumerate(loop):
        #Get data to cuda if possible
        data = data.to(device=device)
        data_CDFI = data_CDFI.to(device = device)
        model = model.to(device=device)

        targets = targets.to(device=device)
        with torch.cuda.amp.autocast():
            scores = model(data,data_CDFI)
            loss = loss_fn(scores,targets)

        losses.append(loss.item())

        #backward
        optimizer.zero_grad()
        scaler.scale(loss).backward() #在16位精度下计算梯度，加速模型计算速度
        scaler.step(optimizer)
        scaler.update()
        loop.set_postfix(loss=loss.item())

    print(f"Loss average over epoch:{sum(losses)/len(losses)}")

def main():

    for nums in range(80):
        save_path = "H:/logs_new/all_classifier_30/"
        breast_path_test = "G:/Multimodal_BR3_classfication/datasets_index/1.txt"
        #breast_path_test = "G:/Multimodal_BR3_classfication/datasets_index/new_follow_up+.txt"
        #breast_path_test_tangshan = "G:/Multimodal_BR3_classfication/datasets_index/new_text_tangshan.txt"
        #breast_path_test_dazhu = "G:/Multimodal_BR3_classfication/datasets_index/new_text_dazhu.txt"

        with open(breast_path_test, "r", encoding='utf-8') as f:
            lines_test = f.readlines()  # 测试集
        np.random.seed(2022)
        np.random.shuffle(lines_train)

        test_ds = Breast_Dataset(
            annotation_lines=np.array(lines_test),
            transform=config.val_transforms,
            train=False,
            maintain_ratio=config.MAINTAIN_RATIO
        )


        test_loader = DataLoader(
            dataset=test_ds, batch_size=config.BATCH_SIZE, num_workers=2, shuffle=False, pin_memory=True,
            drop_last=False
        )
        """
        # 模型选择
        """
        # model =resnet.Multimodel_no_text18(pretrained=True, num_classes=2).cuda()
        model = resnet.Multimodel_no_text50(pretrained=True, num_classes=2).cuda()
        # model = vgg16.Multimodel_VGG(pretrained=True, num_classes=2).cuda()
        # model = SEnet.SE_Multimodel18_no_text(pretrained=True, num_classes=2).cuda()
        # model = SEnet.SE_Multimodel50_no_text(pretrained=True,num_classes=2).cuda()
        # model = Multimodel_efficientNet_v2_s(pretrained=True,num_classes=2).cuda()

        optimizer = optim.Adam(model.parameters(), lr=config.LEARNING_RATE, weight_decay=config.WEIGHT_DECAY)

        # 用于load训练参数
        load_tar = f"H:/logs_new/all_classifier_30/{nums+1}.tar"
        if config.LOAD_MODEL:
            # if config.LOAD_MODEL and config.CHECKPOINT_FILE in os.listdir():
            print("加载好参数")
            load_checkpoint(torch.load(load_tar), model, optimizer, config.LEARNING_RATE)

        accuracy = 0
        recall = 0
        for epoch in range(config.NUM_EPOCHS):
            make_prediction(model=model, loader=test_loader, save_path=save_path, acc=round(accuracy, 3),
                            recall=round(recall, 3), output_csv="epoch_" + str(epoch) + f"test{nums+1}.csv",nums=nums)
            break  # used for test

if __name__ == "__main__":
    main()
