import torch
from torch import nn,optim
import os
import random
import pandas as pd
from sklearn.model_selection import train_test_split
import numpy as np
from baseline.utils import cam_show_img_2
from Nets import resnet
from Nets import SEnet
from Nets import vgg16
from pytorch_grad_cam import GradCAM,ScoreCAM
import baseline.config as config
from torch.utils.data import DataLoader
from tqdm import tqdm
from Nets.EfficienNet import Multimodel_efficientNet_v2_s
from baseline.dataset import Breast_Dataset,Breast_Dataset_with_Path
from torchvision.utils import save_image
from baseline.utils import (
    load_checkpoint,
    save_checkpoint,
    check_accuracy,
    make_prediction,
    make_visualization
)
from PIL import Image

def train_one_epoch(loader , model , optimizer , loss_fn , scaler, device):
    losses = []
    loop = tqdm(loader)
    for batch_index, (data, data_CDFI , targets, _) in enumerate(loop):
        #Get data to cuda if possible
        data = data.to(device=device)
        data_CDFI = data_CDFI.to(device = device)
        model = model.to(device=device)

        targets = targets.to(device=device)
        with torch.cuda.amp.autocast():
            scores = model(data,data_CDFI)
            loss = loss_fn(scores,targets)

        losses.append(loss.item())

        #backward
        optimizer.zero_grad()
        scaler.scale(loss).backward() #在16位精度下计算梯度，加速模型计算速度
        scaler.step(optimizer)
        scaler.update()
        loop.set_postfix(loss=loss.item())

    print(f"Loss average over epoch:{sum(losses)/len(losses)}")

def main():
    """
                    #日志存放路径
    """

    breast_path_test = "G:/Multimodal_BR3_classfication/datasets_index/Ensemble/new_positive.txt"
    # breast_path_test_tangshan = "G:/Multimodal_BR3_classfication/datasets_index/new_text_tangshan.txt"
    # breast_path_test_dazhu = "G:/Multimodal_BR3_classfication/datasets_index/new_text_dazhu.txt"
    with open(breast_path_test, "r", encoding='utf-8') as f:
        lines_tests = f.readlines()  # 测试集

    for a in lines_tests:
        lines_test = []
        lines_test.append(a)
        print(np.array(a))
        test_ds = Breast_Dataset_with_Path(
            annotation_lines=np.array(lines_test),
            transform=config.val_transforms,
            train=False,
            maintain_ratio=config.MAINTAIN_RATIO
        )

        test_loader = DataLoader(
            dataset=test_ds, batch_size=1, num_workers=2, shuffle=False, pin_memory=True,
            drop_last=False
        )
        nums_model = 80
        b_mode_ensemble_val = np.zeros([1024, 12, 12])
        b_mode_ensemble_fmap = np.zeros([1024, 12, 12])
        color_ensemble_val = np.zeros([1024, 12, 12])
        color_ensemble_fmap = np.zeros([1024, 12, 12])

        for nums in range(nums_model):
            print(nums)
            save_path = "H:/logs_new/all_classifier_30/"
            print(lines_test)

            model = resnet.Multimodel_no_text50(pretrained=True, num_classes=2).cuda()

            optimizer = optim.Adam(model.parameters(), lr=config.LEARNING_RATE, weight_decay=config.WEIGHT_DECAY)

            # 用于load训练参数
            load_tar = f"H:/logs_new/all_classifier_30/{nums + 1}.tar"
            if config.LOAD_MODEL:
                # if config.LOAD_MODEL and config.CHECKPOINT_FILE in os.listdir():
                print("加载好参数")
                load_checkpoint(torch.load(load_tar), model, optimizer, config.LEARNING_RATE)

            for epoch in range(config.NUM_EPOCHS):
                grads_val, fmap, c_grads_val, c_fmap = make_visualization(model=model, loader=test_loader,
                                                                          save_path=save_path, nums=nums)
                # print(grads_val)
                b_mode_ensemble_val = grads_val + b_mode_ensemble_val
                b_mode_ensemble_fmap = fmap + b_mode_ensemble_fmap
                color_ensemble_val = c_grads_val + color_ensemble_val
                color_ensemble_fmap = c_fmap + color_ensemble_fmap
                break  # 测试的时候使用

        lesion = lines_test[0].split(';')[-3].split("\n")[0]  # 获得数据的路径
        lesion_withCDFI = lines_test[0].split(';')[-2].split("\n")[0]

        lesion = "G:/Multimodal_BR3_classfication/" + lesion.split('.')[1] + ".tif"
        lesion_withCDFI = "G:/Multimodal_BR3_classfication" + lesion_withCDFI.split('.')[1] + ".tif"

        image = Image.open(lesion).convert("RGB")
        image_resize_180 = image.resize((180, 180), Image.BICUBIC)

        image_CDFI = Image.open(lesion_withCDFI).convert("RGB")
        image_CDFI_resize_180 = image_CDFI.resize((180, 180), Image.BICUBIC)

        image_resize_180 = np.array(image_resize_180)
        image_CDFI_resize_180 = np.array(image_CDFI_resize_180)

        file = lines_test[0].split('/')[3]  # 获取病人的ID号码
        print("平均结果")
        print((b_mode_ensemble_val / (nums_model)))

        cam_show_img_2(image_resize_180, (b_mode_ensemble_fmap / (nums_model)), (b_mode_ensemble_val / (nums_model)),
                       "H:/logs_new/Heatmap", "Ensemble" + "_" + str(file) + "B_")
        cam_show_img_2(image_CDFI_resize_180, (color_ensemble_fmap / (nums_model)), (color_ensemble_val / (nums_model)),
                       "H:/logs_new/Heatmap", "Ensemble" + "_" + str(file) + "C_")

if __name__ == "__main__":
    main()
