import torch
from torch import nn,optim
import os
import random
import pandas as pd
from sklearn.model_selection import train_test_split
import numpy as np
from Nets import resnet
from Nets import SEnet
from Nets import vgg16
import baseline.config as config
from torch.utils.data import DataLoader
from tqdm import tqdm
from Nets.EfficienNet import Multimodel_efficientNet_v2_s

from baseline.dataset import Breast_Dataset
from torchvision.utils import save_image
from baseline.utils import (
    load_checkpoint,
    save_checkpoint,
    check_accuracy,
    make_prediction,
)

def train_one_epoch(loader , model , optimizer , loss_fn , scaler, device):
    losses = []
    loop = tqdm(loader)
    for batch_index, (data, data_CDFI , targets, _) in enumerate(loop):
        #Get data to cuda if possible
        data = data.to(device=device)
        data_CDFI = data_CDFI.to(device = device)
        model = model.to(device=device)

        targets = targets.to(device=device)
        with torch.cuda.amp.autocast():
            scores = model(data,data_CDFI)
            loss = loss_fn(scores,targets)

        losses.append(loss.item())

        #backward
        optimizer.zero_grad()
        scaler.scale(loss).backward() #在16位精度下计算梯度，加速模型计算速度
        scaler.step(optimizer)
        scaler.update()
        loop.set_postfix(loss=loss.item())

    print(f"Loss average over epoch:{sum(losses)/len(losses)}")

def main():
    for i in range(0,100,1):
        save_path = f"H:/logs_new/train{str(i+1)}/"
        svae_path_evaluation = f"H:/logs_new/train{str(i+1)}/evaluation.csv"

        train_positive = "G:/Multimodal_BR3_classfication/datasets_index/Ensemble/train_positive.txt"
        # valid_positive = "G:/Multimodal_BR3_classfication/datasets_index/Ensemble2/valid_positive.txt"

        breast_path_negative = f"G:/Multimodal_BR3_classfication/datasets_index/Ensemble/train{str(i+1)}_4a.txt"
        # breast_path_test = "G:/Multimodal_BR3_classfication/datasets_index/Ensemble/all.txt"

        with open(train_positive, "r", encoding='gbk') as f:
            lines_train_positive = f.readlines()  # 训练集的所有的恶性
        with open(breast_path_negative, "r", encoding='gbk') as f:
            lines_train_negative = f.readlines()  # 训练集的所有的良性
        # np.random.seed(2022)
        # np.random.shuffle(lines_train)

        lines_train = np.append(np.array(lines_train_positive),np.array(lines_train_negative))

        list_targets = []
        for label in np.array(lines_train):
            if str(label).find("BR4") >= 0:
                list_targets.append(int(2))
            else:
                list_targets.append(int(label[0]))

        #print(np.array(lines_train))
        # 训练集留出法留出验证集,分层抽样,均匀的分出3，4a,以及良性


        train_set, valid_set = train_test_split(np.array(lines_train), train_size=0.8, random_state=2022,
                                                stratify=list_targets)  #split validation datasets
        np.random.shuffle(train_set) #shuffle
        """
        #保存训练集与验证集
        """
        print(str(i+1))
        train = open(f'./train_{str(i+1)}.txt', 'w', encoding='utf-8')
        for set in np.array(train_set):
            train.write(set)
        train.close()

        validation = open(f'./validation_{str(i+1)}.txt', 'w', encoding='utf-8')
        for set in np.array(valid_set):
            validation.write(set)
        validation.close()


        #Training
        train_ds = Breast_Dataset(
            annotation_lines=np.array(train_set),
            transform=config.train_transforms,
            maintain_ratio=config.MAINTAIN_RATIO
        )
        #Validation
        val_ds = Breast_Dataset(
            annotation_lines=np.array(valid_set),
            transform=config.val_transforms,
            maintain_ratio=config.MAINTAIN_RATIO
        )

        train_loader = DataLoader(
            dataset=train_ds, batch_size=config.BATCH_SIZE, num_workers=config.NUM_WORKERS, shuffle=True, pin_memory=config.PIN_MEMORY,drop_last=True
        )

        val_loader = DataLoader(
            dataset=val_ds, batch_size=config.BATCH_SIZE, num_workers=config.NUM_WORKERS, shuffle=False, pin_memory=config.PIN_MEMORY,drop_last=True
        )


        loss_fn = nn.CrossEntropyLoss()
        """
        # model selection
        """
        #model =resnet.Multimodel_no_text18(pretrained=True, num_classes=2).cuda()
        model = resnet.Multimodel_no_text50(pretrained=True, num_classes=2).cuda()
        #model = vgg16.Multimodel_VGG(pretrained=True, num_classes=2).cuda()
        #model = SEnet.SE_Multimodel18_no_text(pretrained=True, num_classes=2).cuda()
        #model = SEnet.SE_Multimodel50_no_text(pretrained=True,num_classes=2).cuda()
        #model = Multimodel_efficientNet_v2_s(pretrained=True,num_classes=2).cuda()

        optimizer = optim.Adam(model.parameters(), lr = config.LEARNING_RATE , weight_decay=config.WEIGHT_DECAY)
        #lr_scheduler = optim.lr_scheduler.StepLR(optimizer, step_size=1, gamma=0.95)  # 反向变大再变小是不是更易进入局部最优？

        scaler = torch.cuda.amp.GradScaler()

        precesions = []
        recalls = []
        accuracys = []
        F1s = []
        accuracy = 0
        recall = 0
        for epoch in range(config.NUM_EPOCHS):
            print("第"+str(epoch)+"轮")

            train_one_epoch(train_loader,model, optimizer,loss_fn,scaler,config.DEVICE)  #训练模型

            preds , labels , accuracy,recall, precesion ,F1 = check_accuracy(val_loader,model,config.DEVICE) #验证模型

            accuracys.append(accuracy)
            recalls.append(recall)
            precesions.append(precesion)
            F1s.append(F1)
            print(accuracy)

            #if pre_acc <=accuracy:
                #print("保存模型")
            if config.SAVE_MODEL:
                checkpoint = {
                    "state_dict": model.state_dict(),"optimizer": optimizer.state_dict()
                    }
            save_checkpoint(checkpoint, filename=save_path+"epoch_"+str(epoch)+"Acc_"+str(round(accuracy, 3))+"Rec_"+str(round(recall, 3))+config.CHECKPOINT_FILE)

if __name__ == "__main__":
    main()
