import torch
import pandas as pd
import numpy as np
import baseline.config as config
from pytorch_grad_cam.utils.model_targets import ClassifierOutputTarget
from tqdm import tqdm
from pytorch_grad_cam import GradCAM,ScoreCAM
from pytorch_grad_cam.utils.image import show_cam_on_image
from sklearn.metrics import  roc_auc_score,roc_curve,auc
import matplotlib.pyplot as plt
import cv2
import os

#AUC function
def AUC_curve(label,pred,nums):
    y_1 = np.array(label)
    pred_1 = np.array(pred)
    tprs = []
    aucs = []
    mean_fpr = np.linspace(0, 1, 100)
    plt.figure(figsize=(10, 10))
    #i = 0

    # Compute ROC curve and area the curve
    fpr, tpr, thresholds = roc_curve(y_1, pred_1)
    tprs.append(np.interp(mean_fpr, fpr, tpr))
    tprs[-1][0] = 0.0
    roc_auc = auc(fpr, tpr)
    aucs.append(roc_auc)
    plt.plot(fpr, tpr, lw=1, alpha=0.3,
             label='ROC fold %d (AUC = %0.3f)' % (nums, roc_auc))
    plt.xlim([-0.01, 1.01])
    plt.ylim([-0.01, 1.01])
    plt.xlabel('False Positive Rate', fontsize=18)
    plt.ylabel('True Positive Rate', fontsize=18)
    plt.title('Cross-Validation ROC of Muti-ResNet50', fontsize=18)
    plt.legend(loc="lower right", prop={'size': 15})
    plt.savefig(f"{nums}.png")

# testing
def make_prediction(model,loader ,save_path, output_csv = "sumbmission.csv",acc=None,recall=None,nums=None):
    preds = []
    probs = []
    filenames = []
    labels = []
    model.eval()
    for x , y , label, file in tqdm(loader):
        x = x.to(config.DEVICE)
        y = y.to(config.DEVICE)
        # label = label.to(config.DEVICE)
        with torch.no_grad():  #torch.softmax(model(x,y), dim=-1).cpu().numpy()
            #predicted = model(x,y)
            predicted = torch.softmax(model(x, y), dim=-1)
            pred = predicted.argmax(1)
            prob = predicted[:,1]
            labels.append(label.cpu().numpy())
            preds.append(pred.cpu().numpy())
            probs.append(prob.cpu().numpy())
            filenames += file
    df = pd.DataFrame({"image":filenames,"level":np.concatenate(preds, axis=0),"prob":np.concatenate(probs,axis=0),"label":np.concatenate(labels,axis=0)})
    df.to_csv(save_path+"epoch_"+"Acc_"+str(acc)+"Rec_"+str(recall)+output_csv, index = False)
    #model.train()
    print("Done with predictions")


def cam_show_img_2(img,featuremap,grads,out_dir,file):
    cam = np.zeros(featuremap.shape[1:],dtype=np.float32) #[512, 14, 14] --> (14, 14)
    grads = grads.reshape([grads.shape[0],-1])
    weights = np.mean(grads,axis = 1)
    for i ,w in enumerate(weights):
        cam += w * featuremap[i,:,:]
    cam = np.maximum(cam, 0)
    cam = cam / cam.max()
    cam = cv2.resize(cam, (180, 180))

    heatmap = cv2.applyColorMap(np.uint8(255 * cam), cv2.COLORMAP_JET)
    cam_img = 0.5 * heatmap + img

    path_cam_img = os.path.join(out_dir, file+".jpg")
    cv2.imwrite(path_cam_img, cam_img)

def cam_show_img(img,featuremap,grads,out_dir,file):
    cam = np.zeros(featuremap.shape[1:], dtype=np.float32)  # [512, 14, 14] --> (14, 14)
    grads = grads.reshape([grads.shape[0], -1])
    weights = np.mean(grads, axis=1)
    for i, w in enumerate(weights):
        cam += w * featuremap[i, :, :]
    cam = np.maximum(cam, 0)
    cam = cam / cam.max()
    cam = cv2.resize(cam, (180, 180))

    heatmap = cv2.applyColorMap(np.uint8(255 * cam), cv2.COLORMAP_JET)
    cam_img = 0.5 * heatmap + img.squeeze(dim=0).cpu().numpy()

    path_cam_img = os.path.join(out_dir, file + ".jpg")
    cv2.imwrite(path_cam_img, cam_img)

def backward_hook(module, grad_in, grad_out):
    bmode_grad_block.append(grad_out[0].detach())

def farward_hook(module, input, output):
    bmode_fmap_block.append(output)

def backward_hook_(module, grad_in, grad_out):
    color_grad_block.append(grad_out[0].detach())

def farward_hook_(module, input, output):
    color_fmap_block.append(output)

bmode_fmap_block = list()
bmode_grad_block = list()

color_fmap_block = list()
color_grad_block = list()


def make_visualization(model,loader ,save_path,nums):

    model.eval()
    for x , y , label, file,path_b_mode, path_color in tqdm(loader):
        x = x.to(config.DEVICE)
        y = y.to(config.DEVICE)
        # label = label.to(config.DEVICE)
        #with torch.no_grad():  #torch.softmax(model(x,y), dim=-1).cpu().numpy()
        if 1==1:
            #注册hook
            model.model.layer3.register_forward_hook(farward_hook)  # layer3:1024*12*12 layer4:2048*6*6
            model.model.layer3.register_backward_hook(backward_hook)

            #forward
            # predicted = model(x,y)
            predicted = torch.softmax(model(x, y), dim=-1)
            pred = predicted.argmax(1)  # 取预测概率最大的结果
            if pred == 1:
                class_name = "恶性"
            else:
                class_name = "良性"

            #backforward
            print(class_name)
            model.zero_grad()
            class_loss = model(x, y)[0,1]
            class_loss.requires_grad_(True)
            class_loss.backward()

            grads_val = bmode_grad_block[0].cpu().data.numpy().squeeze()
            fmap = bmode_fmap_block[0].cpu().data.numpy().squeeze()

            cam_show_img(path_b_mode, fmap, grads_val, "H:/logs_new/Heatmap",str(nums)+"_"+str(file[0])+"_"+str(pred.item())+"_"+'{:.4f}'.format(predicted.max().item())+"B_")
            bmode_fmap_block.clear()
            bmode_grad_block.clear()

        if 1==1:
            #注册hook
            model.model_CDIF.layer3.register_forward_hook(farward_hook_)  # 9
            model.model_CDIF.layer3.register_backward_hook(backward_hook_)
            #forward
            # predicted = model(x,y) #按照列最大值的索引
            predicted = torch.softmax(model(x, y), dim=-1)
            pred = predicted.argmax(1)  # 取预测概率最大的结果
            if pred == 1:
                class_name = "恶性"
            else:
                class_name = "良性"
            #backforward
            print(class_name)
            model.zero_grad()
            class_loss = model(x, y)[0,1]
            class_loss.requires_grad_(True)
            class_loss.backward()

            c_grads_val = color_grad_block[0].cpu().data.numpy().squeeze()
            c_fmap = color_fmap_block[0].cpu().data.numpy().squeeze()

            cam_show_img(path_color, c_fmap, c_grads_val, "H:/logs_new/Heatmap",str(nums)+"_"+str(file[0])+"_"+str(pred.item())+"_"+'{:.4f}'.format(predicted.max().item())+"C_")

            color_fmap_block.clear()
            color_grad_block.clear()

    print("Done with predictions")
    return grads_val,fmap,c_grads_val,c_fmap

def check_accuracy(loader, model , device = "cuda"):

    model.eval()
    #loop = tqdm(loader)
    all_preds, all_labels = [] , []
    num_correct = 0
    num_samples = 0
    num_positive = 0
    num_false_negative = 0
    num_false_positive = 0
    for x , y , label, _ in loader:
        x = x.to(device=device)
        y = y.to(device = device)
        label = label.to(device = device)
        with torch.no_grad():
            scores = model(x,y)
            predicted = torch.softmax(scores, dim=-1)
            prob = predicted[:, 1]
            _, predictions = scores.max(1)

            #所有准确率
            num_correct += (predictions==label).sum()
            num_samples += predictions.shape[0]

            #阳性准确率
            num_positive += ((predictions + label)==2).sum() #真正阳性的个数

            #假阴的个数
            num_false_negative +=((label-2*predictions)==1).sum() #假阴性个数 label=1,predict=0 假阴

            #假阳的个数
            num_false_positive += ((label-2*predictions)==-2).sum()  #label=0,prediction==1
            all_preds.append(prob.detach().cpu().numpy())
            all_labels.append(label.detach().cpu().numpy())

    print(
            f"Got {num_correct}/{num_samples} with accuracy {float(num_correct)/float(num_samples) *100:.2f}"
        )
    accuracy = (float(num_correct))/float(num_samples) * 100
    recall = (float(num_positive))/(float(num_positive) + float(num_false_negative)+1) * 100
    precesion = (float(num_positive))/(float(num_positive) + float(num_false_positive)+1)*100
    #run_biopsy = (float(num_false_positive))/(float(num_positive))
    F1 = (2*recall*precesion)/(precesion+recall+1)
    #model.train()
    #return np.concatenate(all_preds,axis=0), np.concatenate(all_labels,axis=0) , accuracy, recall, precesion ,F1, run_biopsy
    return np.concatenate(all_preds, axis=0), np.concatenate(all_labels,axis=0), accuracy, recall, precesion, F1

def save_checkpoint(state, filename = "my_checkpoint.pth.tar"):
    print("=>Saveing checkpoing")
    torch.save(state, filename)

def load_checkpoint(checkpoint,model,optimizer,lr):
    print("=>Loading checkpoing")
    model.load_state_dict(checkpoint["state_dict"])
    optimizer.load_state_dict(checkpoint["optimizer"])
    for state in optimizer.state.values():
        for k, v in state.items():
            if torch.is_tensor(v):
                state[k] = v.cuda()
    for param_group in optimizer.param_groups:
        param_group["lr"] = lr