import torch
import torch.nn as nn
#from torchvision.models.utils import load_state_dict_from_url
from torch.hub import load_state_dict_from_url
model_urls = {
    'vgg16': 'https://download.pytorch.org/models/vgg16-397923af.pth',
}


class VGG(nn.Module):
    def __init__(self, features, init_weights=True):
        super(VGG, self).__init__()
        self.features = features
        self.gap = nn.AdaptiveAvgPool2d((1, 1))
        # self.classifier = nn.Sequential(
        #     nn.Linear(512 * 7 * 7, 4096),
        #     nn.ReLU(True),
        #     nn.Dropout(),
        #     nn.Linear(4096, 4096),
        #     nn.ReLU(True),
        #     nn.Dropout(),
        #     nn.Linear(4096, num_classes),
        # )
        if init_weights:
            self._initialize_weights()

    def forward(self, x):
        x = self.features(x)
        x = self.gap(x)
        x = torch.flatten(x, 1)
        #x = self.classifier(x)
        return x

    def _initialize_weights(self):
        for m in self.modules():
            if isinstance(m, nn.Conv2d):
                nn.init.kaiming_normal_(m.weight, mode='fan_out', nonlinearity='relu')
                if m.bias is not None:
                    nn.init.constant_(m.bias, 0)
            elif isinstance(m, nn.BatchNorm2d):
                nn.init.constant_(m.weight, 1)
                nn.init.constant_(m.bias, 0)
            elif isinstance(m, nn.Linear):
                nn.init.normal_(m.weight, 0, 0.01)
                nn.init.constant_(m.bias, 0)
    
    def freeze_backbone(self):
        for param in self.features.parameters():
            param.requires_grad = False

    def Unfreeze_backbone(self):
        for param in self.features.parameters():
            param.requires_grad = True


def make_layers(cfg, batch_norm=False):
    layers = []
    in_channels = 3
    for v in cfg:
        if v == 'M':
            layers += [nn.MaxPool2d(kernel_size=2, stride=2)]
        else:
            conv2d = nn.Conv2d(in_channels, v, kernel_size=3, padding=1)
            if batch_norm:
                layers += [conv2d, nn.BatchNorm2d(v), nn.ReLU(inplace=True)]
            else:
                layers += [conv2d, nn.ReLU(inplace=True)]
            in_channels = v
    return nn.Sequential(*layers)

# 224,224,3 -> 224,224,64 -> 112,112,64 -> 112,112,128 -> 56,56,128 -> 56,56,256 -> 28,28,256 -> 28,28,512
# 14,14,512 -> 14,14,512 -> 7,7,512
cfgs = {
    'D': [64, 64, 'M', 128, 128, 'M', 256, 256, 256, 'M', 512, 512, 512, 'M', 512, 512, 512, 'M'],
}

def vgg16(pretrained=False, progress=True):
    model = VGG(make_layers(cfgs['D']))
    if pretrained:
        state_dict = load_state_dict_from_url(model_urls['vgg16'], model_dir='./model_data',
                                              progress=progress)
        model.load_state_dict(state_dict,strict=False)

    # if num_classes!=1000:
    #     model.classifier =  nn.Sequential(
    #         nn.Linear(512 * 7 * 7, 4096),
    #         nn.ReLU(True),
    #         nn.Dropout(),
    #         nn.Linear(4096, 4096),
    #         nn.ReLU(True),
    #         nn.Dropout(),
    #         nn.Linear(4096, num_classes),
    #     )
    return model


class Multimodel_VGG(nn.Module):
    def __init__(self,pretrained,num_classes):
        super(Multimodel_VGG, self).__init__()
        self.num_classes = num_classes
        self.model = vgg16(pretrained=pretrained,progress=True)
        self.model_CDIF = vgg16(pretrained=pretrained,progress=True)
        self.fc = nn.Sequential(
            nn.Linear(1024, 256),
            nn.ReLU(),
            nn.Dropout(0.5),
            nn.Linear(256, self.num_classes)
        )
    def forward(self,x,x_withCDFI):
        x = self.model(x)
        x = torch.flatten(x, 1)
        x_withCDFI = self.model_CDIF(x_withCDFI)
        x_withCDFI = torch.flatten(x_withCDFI, 1)
        output = torch.cat((x, x_withCDFI), 1)
        output = self.fc(output)
        return output

    def Multi_freeze_backbone(self):
        self.model.freeze_backbone()
        self.model_CDIF.freeze_backbone()

    def Multi_Unfreeze_backbone(self):
        self.model.Unfreeze_backbone()
        self.model_CDIF.Unfreeze_backbone()



if __name__ == "__main__":
    vgg16 = Multimodel_VGG(pretrained=False,num_classes=2)
    input = torch.randn( 4, 3, 200, 200)
    input2 = torch.randn(4, 3, 200, 200)
    vgg16.Multi_freeze_backbone()
    output = vgg16(input,input2)
    print(output.shape)
